@@ -5,7 +5,7 @@
  * Author:   Georg Fleischmann
  *
  * created:  1998-12-19
- * modified: 2005-09-10
+ * modified: 2005-10-20
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -36,7 +36,7 @@
 #include "AstroChart.h"
 #include "astroCommon.h"
 #include "TopocentricCycle.h"
-#include "Waves.h"
+//#include "Waves.h"
 #include "SwissEphemeris.subproj/Ephemeris.h"
 #include "Astro.bproj/AstroController.h"
 
@@ -250,9 +250,15 @@ static NSArray *colorWheel(float s)
 }
 
 /* scale, degrees */
+typedef enum
+{
+    SCALEDEG_NONE = 0,	// no scale
+    SCALEDEG_360  = 1,	// 360 degree
+    SCALEDEG_DEC  = 2	// declination
+}ScaleDegreeType;
 - (VGroup*)scaleWithCenter:(NSPoint)center radius:(float)radius
                 scaleSize1:(float)scaleSize1 scaleSize2:(float)scaleSize2
-               showDegrees:(BOOL)showDegrees rotate:(float)angle
+                degreeType:(ScaleDegreeType)degreeType rotate:(float)angle
 {   int		i;
     VLine	*line;
     VGroup	*group = [VGroup group];
@@ -262,7 +268,7 @@ static NSArray *colorWheel(float s)
     for (i=0; i<360; i++)
     {
         p0 = NSMakePoint(center.x+radius - ((i%5) ? scaleSize1 : scaleSize2), center.y);
-        if (!showDegrees && i%30 == 0)	// longer scales every 30 degrees without text
+        if (!degreeType && i%30 == 0)	// longer scales every 30 degrees without text
             p0.x -= scaleSize1;
         p1 = NSMakePoint(center.x+radius, center.y);
         line = [VLine lineWithPoints:p0 :p1];
@@ -270,11 +276,24 @@ static NSArray *colorWheel(float s)
         //[line setColor:color];
         [group addObject:line];
 
-        if ( showDegrees && !(i%30) )
-            [group addObject:[view text:[NSString stringWithFormat:@"%d", i]
+        if ( degreeType && !(i%30) )
+        {   int	i1 = i;
+
+            if (degreeType == SCALEDEG_DEC)
+            {
+                if (i1 > 180 && i1 <= 360)	// 3Q, 4Q -> 1Q, 2Q
+                    i1 = 360 - i1;
+                if (i1 > 90 && i1 <= 180)	// 2Q, (3Q, 4Q) -> 1Q
+                    i1 = 180 - i1;
+                i1 /= 3;			// display 10, 20, 30
+                if (i > 180)			// 2Q, 3Q
+                    i1 = -i1;
+            }
+            [group addObject:[view text:[NSString stringWithFormat:@"%d", i1]
                                  center:NSMakePoint(p0.x-scaleSize1, p0.y)
                                   color:[NSColor blackColor]
                                   angle:angle+i center:center font:systemFont8]];
+        }
     }
     return group;
 }
@@ -331,10 +350,12 @@ static NSArray *colorWheel(float s)
     return nil;
 }
 
+/* modified: 2005-10-19 (decDict added)
+ */
 - (VGroup*)horoscopeWithCenter:(NSPoint)cp radius:(float)radius tableOrigin:(NSPoint)tableOrigin
                       template:(NSArray*)template color:(NSColor*)color title:(NSString*)title
                            lat:(float)latitude lon:(float)longitude city:(NSString*)city gmt:(NSCalendarDate*)gmt
-{   NSDictionary	*degreeDict, *nodeDict, *speedLonDict;
+{   NSDictionary	*degreeDict, *nodeDict, *speedLonDict, *decDict, *speedDecDict;
     NSMutableArray	*planets = [NSMutableArray arrayWithObjects:@"AC", @"MC", nil];
     NSArray		*nodes = Prefs_Nodes;
     NSPoint		p0, p1;
@@ -356,11 +377,14 @@ static NSArray *colorWheel(float s)
     [planets addObjectsFromArray:Prefs_Objects];
 
     [ephemeris setMeanNodes:Prefs_MeanNodes];
-    degreeDict = [ephemeris objectDictAtUTC:gmt lat:latitude lon:longitude elev:0.0 topocentric:Prefs_Topocentric];
+    degreeDict   = [ephemeris objectDictAtUTC:gmt lat:latitude lon:longitude elev:0.0
+                                  topocentric:Prefs_Topocentric];
     if (Prefs_ACFix)
         rotateAC = 180.0 - [degreeDict floatForKey:@"AC"];	// rotation of chart
-    nodeDict = [ephemeris currentNodeDict];
+    nodeDict     = [ephemeris currentNodeDict];
     speedLonDict = [ephemeris currentSpeedLonDict];
+    decDict      = [ephemeris currentDeclinationDict];
+    speedDecDict = [ephemeris currentSpeedDecDict];
 
     /* background color */
     //[view setBackgroundColor:[NSColor colorWithCalibratedRed:1.0 green:1.0 blue:0.87 alpha:1.0]];
@@ -386,7 +410,7 @@ static NSArray *colorWheel(float s)
     /* scale */
     /*[group addObject:[self scaleWithCenter:cp radius:radius*RADIUS_2
                                 scaleSize1:radius*SCALE_MIN scaleSize2:radius*SCALE_MAX
-                               showDegrees:YES]];*/
+                                degreeType:SCALEDEG_360]];*/
 
     /* local scale */
     if (Prefs_LocalScale)
@@ -439,6 +463,49 @@ static NSArray *colorWheel(float s)
                               color:color angle:angle center:cp font:planetFont]];
     }
 
+    /* planet declination */
+    if (Prefs_ShowDeclination)
+    {   //BOOL	retro = NO;
+        VArc	*arc = [VArc arc];
+
+        [arc setCenter:cp start:NSMakePoint(cp.x+radius*RADIUS_3, cp.y) angle:360.0];
+        [group addObject:arc];
+        [group addObject: [self scaleWithCenter:cp radius:radius*RADIUS_3
+                                     scaleSize1:radius*SCALE_MIN scaleSize2:radius*SCALE_MAX
+                                     degreeType:SCALEDEG_DEC rotate:rotateAC]];
+        for ( i=2; i<(int)[planets count]; i++ )
+        {   NSString	*planet = [planets objectAtIndex:i];
+            float	dec  = [decDict      floatForKey:planet] * 3.0;
+            float	lon  = [degreeDict   floatForKey:planet];
+            float	vDec = [speedDecDict floatForKey:planet];
+            //float	vLon = [speedLonDict floatForKey:planet];
+            //BOOL	flip = (vDec * vLon < 0.0) ? YES : NO;	// (vDec < 0.0 && vLon >= 0.0);
+            BOOL	flip = ( lon >= 90.0 && lon <= 270.0 ) ? YES : NO;
+
+            /* we place the declination symbol on the side of the longitude.
+             * We add an R to the declination, if the direction is counter the direction of the side
+             */
+            if (flip)	// display retrograde on the other side
+                dec = mod360(180.0 - dec);
+            //if (rotateAC)
+            //    dec = standardAngle(dec + rotateAC);
+            p0 = NSMakePoint(cp.x + radius*RADIUS_3 - radius*(SCALE_MAX+SCALE_MIN), cp.y);
+            p1 = NSMakePoint(cp.x + radius*RADIUS_3 /*+ radius*SCALE_MIN*/, cp.y);
+            line = [VLine lineWithPoints:p0 :p1];
+            [line setAngle:-dec withCenter:cp];
+            [line setColor:color];
+            [group addObject:line];
+
+            /* retrograde means against the regular direction */
+            string = ( ((vDec < 0.0 && !flip) || (vDec > 0.0 && flip)) )
+                     ? [NSString stringWithFormat:@"%@X", planetSymbol(planet)]
+                     : planetSymbol(planet);
+            [group addObject:[view text:string
+                                 center:NSMakePoint(p0.x-(radius*SCALE_MAX), cp.y)
+                                  color:color angle:dec center:cp font:planetFont]];
+        }
+    }
+
     /* AC / DC */
     /*{
         ac = [self ac:sideralTime latitude:latitude];
@@ -991,7 +1058,7 @@ static NSArray *colorWheel(float s)
 
 /* Tansite aspect table between horoscope and composite
  * created:  2005-07-03
- * modified: 2005-09-10 (show degrees only)
+ * modified: 2005-09-19 (show symbol row on the left for symmetry)
  * Author:  Ilonka Fleischmann
  */
 - (VGroup*)compositeTable:(NSPoint)cp tableOrigin:(NSPoint)tableOrigin
@@ -1011,6 +1078,7 @@ static NSArray *colorWheel(float s)
     VText		*text;
     int			i, j, cnt;
     BOOL		showCombiAspects = NO;	// YES = show inner/outer + outer/inner
+    BOOL		show2ndSymbolRow = NO;	// YES displays 2nd row of symbols to the right of degrees
 
     speedLonDict = [ephemeris currentSpeedLonDict];
 
@@ -1061,7 +1129,10 @@ static NSArray *colorWheel(float s)
             if (showCombiAspects)	// show aspect table
                 x = tableOrigin.x - ([planetsH count]*cellSize + cellSize + [text bounds].size.width + 6.0);
             else			// show degrees only
-                x = tableOrigin.x - (cellSize + [text bounds].size.width + 6.0);
+            {   x = tableOrigin.x - ([text bounds].size.width + 6.0);
+                if ( show2ndSymbolRow )	// room for 2nd row of symbols
+                    x -= cellSize;
+            }
 
             /* move text to real origin */
             [text moveBy:NSMakePoint((x + 3.0) - r.origin.x, 0.0)];
@@ -1090,36 +1161,75 @@ static NSArray *colorWheel(float s)
             x += w;
         }
 
-        /* composite symbols vertical (2nd column) */
-        [mString setString:@""];
-        for ( i=0, cnt=(int)[planets count]; i<cnt; i++ )
-            [mString appendFormat:@"%@%@", planetSymbol([planets objectAtIndex:i]), (i<cnt-1) ? @"\n" : @""];
-        text = [view text:mString base:NSMakePoint(x, y) color:color angle:0.0
-                   center:cp lineHeight:cellSize font:symbolFont];
-        r = [text bounds];
-#ifdef __APPLE__
-        if (r.size.height > cnt * cellSize + 2)	// workaround Apple -> reduce lineHeight
-        {   text = [view text:mString base:NSMakePoint(x, y) color:color angle:0.0
-                       center:cp lineHeight:cellSize-1 font:symbolFont];
-            r = [text bounds];
-        }
-#endif
-        [text moveTo:NSMakePoint(r.origin.x + (cellSize - r.size.width)/2.0, y)];
-        [subGroup addObject:text];
-
-        /* horicontal lines for planet symbols */
-        for (i=0; i<=(int)[planets count] /*+1*/; i++ )
+        /* symbols vertical (1st leading column) */
+        if ( !showCombiAspects )
         {
-            p0 = NSMakePoint(x, y + i * cellSize);
-            p1 = NSMakePoint(x + cellSize, p0.y);
+            x -= w + cellSize;
+            [mString setString:@""];
+            for ( i=0, cnt=(int)[planets count]; i<cnt; i++ )
+                [mString appendFormat:@"%@%@", planetSymbol([planets objectAtIndex:i]), (i<cnt-1) ? @"\n" : @""];
+            text = [view text:mString base:NSMakePoint(x, y) color:color angle:0.0
+                       center:cp lineHeight:cellSize font:symbolFont];
+            r = [text bounds];
+#ifdef __APPLE__
+            if (r.size.height > cnt * cellSize + 2)	// workaround Apple -> reduce lineHeight
+            {   text = [view text:mString base:NSMakePoint(x, y) color:color angle:0.0
+                           center:cp lineHeight:cellSize-1 font:symbolFont];
+                r = [text bounds];
+            }
+#endif
+            [subGroup addObject:text];
+            [text moveTo:NSMakePoint(r.origin.x + (cellSize - r.size.width)/2.0, y)];
+
+            /* horicontal lines for planet symbols */
+            for (i=0; i<=(int)[planets count] /*+1*/; i++ )
+            {
+                p0 = NSMakePoint(x, y + i * cellSize);
+                p1 = NSMakePoint(x + cellSize, p0.y);
+                line = [VLine lineWithPoints:p0 :p1];
+                [subGroup addObject:line];
+            }
+            /* vertical line for planet symbols */
+            p0 = NSMakePoint(x, y);
+            p1 = NSMakePoint(p0.x, y + [planets count] * cellSize);
+            line = [VLine lineWithPoints:p0 :p1];
+            [subGroup addObject:line];
+
+            x += w + cellSize;
+        }
+        if ( showCombiAspects || show2ndSymbolRow )
+        {
+            /* composite symbols vertical (2nd column) */
+            [mString setString:@""];
+            for ( i=0, cnt=(int)[planets count]; i<cnt; i++ )
+                [mString appendFormat:@"%@%@", planetSymbol([planets objectAtIndex:i]), (i<cnt-1) ? @"\n" : @""];
+            text = [view text:mString base:NSMakePoint(x, y) color:color angle:0.0
+                       center:cp lineHeight:cellSize font:symbolFont];
+            r = [text bounds];
+#ifdef __APPLE__
+            if (r.size.height > cnt * cellSize + 2)	// workaround Apple -> reduce lineHeight
+            {   text = [view text:mString base:NSMakePoint(x, y) color:color angle:0.0
+                           center:cp lineHeight:cellSize-1 font:symbolFont];
+                r = [text bounds];
+            }
+#endif
+            [text moveTo:NSMakePoint(r.origin.x + (cellSize - r.size.width)/2.0, y)];
+            [subGroup addObject:text];
+
+            /* horicontal lines for planet symbols */
+            for (i=0; i<=(int)[planets count] /*+1*/; i++ )
+            {
+                p0 = NSMakePoint(x, y + i * cellSize);
+                p1 = NSMakePoint(x + cellSize, p0.y);
+                line = [VLine lineWithPoints:p0 :p1];
+                [subGroup addObject:line];
+            }
+            /* vertical line for planet symbols */
+            p0 = NSMakePoint(x+cellSize, y);
+            p1 = NSMakePoint(p0.x, y + [planets count] * cellSize);
             line = [VLine lineWithPoints:p0 :p1];
             [subGroup addObject:line];
         }
-        /* vertical line for planet symbols */
-        p0 = NSMakePoint(x+cellSize, y);
-        p1 = NSMakePoint(p0.x, y + [planets count] * cellSize);
-        line = [VLine lineWithPoints:p0 :p1];
-        [subGroup addObject:line];
 
         if ( showCombiAspects )
         {
@@ -1396,7 +1506,7 @@ static NSArray *colorWheel(float s)
     NSPoint	tableOrigin = NSMakePoint(20.0, 37.0), tableOriginC = NSMakePoint(573.0, 37.0);
     NSPoint	start, startComposite;
     float	radius = 204.0, radiusComposite = 144.0, rotateAC = 0.0;
-    int		ixMaskRadix = -1, ixRadix = 0, ixTemplate = 2, ixWaves = -1, i;
+    int		ixMaskRadix = -1, ixRadix = 0, ixTemplate = 2/*, ixWaves = -1*/, i;
     NSArray	*template = nil;	// list of marks and text showing positions
 
     if (!gmt || ![[[view window] title] hasPrefix:CHART_PREFIX])
@@ -1435,8 +1545,8 @@ static NSArray *colorWheel(float s)
         ixRadix = 1;
         ixMaskComposite = 2;
         ixComposite = 3;
-        if ([layerList count] > 5)
-            ixWaves = 4;
+        //if ([layerList count] > 5)
+        //    ixWaves = 4;
         ixTemplate = [layerList count]-1;
         template = [[layerList objectAtIndex:ixTemplate] list];
         [self getMarksFromTemplate:template
@@ -1477,7 +1587,7 @@ static NSArray *colorWheel(float s)
         /* scale */
         [layer addObject:[self scaleWithCenter:cp radius:radius*RADIUS_2
                                     scaleSize1:radius*SCALE_MIN scaleSize2:radius*SCALE_MAX
-                                   showDegrees:YES rotate:rotateAC]];
+                                    degreeType:SCALEDEG_360 rotate:rotateAC]];
     }
     /* rotate AC to the left
      * we create a rotated copy of the mask and turn off the mask
@@ -1506,6 +1616,26 @@ static NSArray *colorWheel(float s)
     //[layer addObject:g];
     [layer addObjectsFromArray:[(VGroup*)g list]];
 
+    /* give other modules a chance to add their stuff
+     */
+    {   NSDictionary	*infoDict = nil;
+
+        infoDict = [NSDictionary dictionaryWithObjectsAndKeys:
+                    layerList,                                  @"layerList",
+                    layer,                                      @"layerChart",
+                    [NSNumber numberWithInt:rotateAC],          @"rotateAC",
+                    gmt,                                        @"date",
+                    [NSNumber numberWithFloat:latitude],        @"lat",
+                    [NSNumber numberWithFloat:longitude],       @"lon",
+                    propertyListFromNSPoint(cp),                @"center",
+                    [NSNumber numberWithFloat:radius],          @"radius",
+                    [NSNumber numberWithFloat:radius*RADIUS_3], @"radiusComposite",
+                    [NSNumber numberWithFloat:radius*RADIUS_4], @"radiusStar", nil];
+        [[NSNotificationCenter defaultCenter] postNotificationName:AstroAddChartParts
+                                                            object:self userInfo:infoDict];
+    }
+
+#if 0
     /* waves */
     if (ixWaves >= 0)
         layer = [layerList objectAtIndex:ixWaves];
@@ -1526,7 +1656,7 @@ static NSArray *colorWheel(float s)
                             center:cp radius:radius amplitude:5.0];
         [layer addObjectsFromArray:[(VGroup*)g list]];
     }
-
+#endif
 
     /* composite */
     if (composite)
@@ -1574,7 +1704,7 @@ static NSArray *colorWheel(float s)
             /* scale */
             [layer addObject:[self scaleWithCenter:cp radius:radius*RADIUS_3
                                         scaleSize1:radius*SCALE_MIN scaleSize2:radius*SCALE_MAX
-                                       showDegrees:NO rotate:rotateAC]];
+                                        degreeType:SCALEDEG_NONE rotate:rotateAC]];
         }
         /* rotate AC to the left
          * we create a rotated copy of the mask and turn off the mask
