@@ -1,11 +1,11 @@
 /* EventManager.m
  * controller class and files owner of the EventPanel
  *
- * Copyright (C) 2003-2004 by vhf interservice GmbH
+ * Copyright (C) 2003-2005 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2003-06-22
- * modified: 2004-10-06
+ * modified: 2005-11-16 (use vhfPathWithPathComponents())
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -31,6 +31,7 @@
 #include "../Cenon/App.h"
 #include "../Cenon/functions.h"
 #include "../Cenon/messages.h"
+#include "astroLocations.h"
 #include "astroMessages.h"
 #include "AstroPrincipal.h"
 #include "Astro.bproj/AstroController.h"
@@ -210,6 +211,9 @@ static int indexFromType(NSString *typeString)
     [[Astro_Principal timeZoneManager] showPanel:self];
 }
 
+/* add new entry
+ * modified: 2005-11-13
+ */
 - (void)add:sender
 {   NSMutableDictionary	*event = [NSMutableDictionary dictionary];
     int			folderIx = [eventBrowser selectedRowInColumn:0], eventIx, i;
@@ -238,9 +242,21 @@ static int indexFromType(NSString *typeString)
         }
     }
 
-    /* add data fields */
-    if (![eventField stringValue])
+    if (![eventField stringValue])	// empty title
         return;
+
+    /* name already exists -> clear fields (?) */
+    for (i=0; i<[events count]; i++)
+    {   NSString	*name = [[events objectAtIndex:i] objectForKey:@"name"];
+
+        if ( [[eventField stringValue] isEqual:name] )
+        {
+            [self clear:self];
+            return;
+        }
+    }
+
+    /* add data fields */
     [event setObject:[eventField stringValue] forKey:@"name"];
     if ([[dateField stringValue] length] == 10 && [[timeField stringValue] length] == 5)
     {   NSString	*tzStr = [tzField stringValue];
@@ -433,29 +449,139 @@ static int indexFromType(NSString *typeString)
     [self saveEvents];
 }
 
-/* 
+/* returns the current event list or nil if none
+ * created: 2005-08-31
+ */
+- (NSArray*)selectedEventList
+{   int		folderIx = [eventBrowser selectedRowInColumn:0];
+    int		eventIx  = [eventBrowser selectedRowInColumn:1];
+    NSArray	*events;
+
+    if (folderIx < 0)
+        return nil;
+
+    events = [eventDict objectForKey:[folders objectAtIndex:folderIx]];
+    if (!events)
+        events = [extEventDict objectForKey:[folders objectAtIndex:folderIx]];
+    if ([events isKindOfClass:[NSDictionary class]])	// sub-folders
+    {   NSDictionary	*subEvents;
+        int		subFolderIx = eventIx;
+
+        //eventIx = [eventBrowser selectedRowInColumn:2];
+        //if (eventIx < 0)
+        //    return nil;
+        subEvents = [extEventDict objectForKey:[folders objectAtIndex:folderIx]];		// Met.db
+        events = [subEvents objectForKey:[[subEvents allKeys] objectAtIndex:subFolderIx]];	// List of Rain
+    }
+    return events;
+}
+/* returns event row for a selected folder
+ * created: 2005-08-31
+ */
+- (int)eventRow
+{   int		folderIx = [eventBrowser selectedRowInColumn:0];
+    NSArray	*events;
+
+    if (folderIx < 0)
+        return -1;
+    events = [eventDict objectForKey:[folders objectAtIndex:folderIx]];
+    if (!events)
+        events = [extEventDict objectForKey:[folders objectAtIndex:folderIx]];
+    if ([events isKindOfClass:[NSDictionary class]])	// sub-folders
+        return 2;
+    return 1;
+}
+
+/* fills the event fields from an event dictionary
+ * created: 2005-08-31
+ */
+- (void)setEvent:(NSDictionary*)event
+{   NSString	*string;
+
+    if (!event)
+    {   [self clear:self];
+        return;
+    }
+    [eventField setStringValue:[event objectForKey:@"name"]];
+    if ( (string = [event stringForKey:@"date"]) )
+    {   NSCalendarDate	*date;
+
+        if ( isdigit([string characterAtIndex:[string length]-1]) )
+        {   date = [NSCalendarDate dateWithString:string calendarFormat:@"%Y-%m-%d %H:%M %z"];
+            [tzField setStringValue:[date descriptionWithCalendarFormat:@"%z"]];
+        }
+        else
+        {   date = [NSCalendarDate dateWithString:string calendarFormat:@"%Y-%m-%d %H:%M %Z"];
+            [tzField setStringValue:[date descriptionWithCalendarFormat:@"%Z"]];
+        }
+        [dateField setStringValue:[date descriptionWithCalendarFormat:Prefs_DateFormat]];
+        [timeField setStringValue:[date descriptionWithCalendarFormat:@"%H:%M"]];
+    }
+    else
+    {   [dateField setStringValue:@""];
+        [timeField setStringValue:@""];
+        [tzField setStringValue:@"+0000"];
+    }
+    [typePopup selectItemAtIndex:indexFromType([event objectForKey:@"type"])];
+    [countryField setStringValue:(string=[event objectForKey:@"country"]) ? string : @""];
+    [cityField    setStringValue:(string=[event objectForKey:@"city"])    ? string : @""];
+    [latField     setStringValue:(string=[event stringForKey:@"lat"])     ? string : @""];
+    [lonField     setStringValue:(string=[event stringForKey:@"lon"])     ? string : @""];
+    [textView     setString:     (string=[event objectForKey:@"text"])    ? string : @""];
+}
+
+/* search for an event (event name only)
+ * created: 2005-08-31
  */
 - (void)search:sender
-{
-    NSLog(@"Search: eventPanel=%@ eventBrowser=%@", eventPanel, eventBrowser);
+{   int		i, eventIx = -1;
+    NSString	*string;
+
+    if ([string=[eventField stringValue] length])
+    {   NSArray	*events = [self selectedEventList];
+
+        for (i=0; i<[events count]; i++)
+        {   NSString	*name = [[events objectAtIndex:i] objectForKey:@"name"];
+
+            if ([name rangeOfString:string].length)
+            {   eventIx = i;
+                [eventBrowser selectRow:eventIx inColumn:[self eventRow]];
+                [self setEvent:[events objectAtIndex:eventIx]];
+                break;
+            }
+        }
+    }
 }
 
 /* set selection in Chart-Panel etc. via notification
+ * modified: 2005-07-25
  */
 - (void)set:sender
 {   int			folderIx = [eventBrowser selectedRowInColumn:0];
     int			eventIx  = [eventBrowser selectedRowInColumn:1];
     NSArray		*events;
-    NSDictionary	*event;
+    NSDictionary	*event, *infoDict = nil;
 
     if (eventIx < 0)
         return;
     events = [eventDict objectForKey:[folders objectAtIndex:folderIx]];
     if (!events)
         events = [extEventDict objectForKey:[folders objectAtIndex:folderIx]];
+    if ([events isKindOfClass:[NSDictionary class]])	// sub-folders
+    {   NSDictionary	*subEvents;
+        int		subFolderIx = eventIx;
+
+        eventIx = [eventBrowser selectedRowInColumn:2];
+        if (eventIx < 0)
+            return;
+        subEvents = [extEventDict objectForKey:[folders objectAtIndex:folderIx]];		// Met.db
+        events = [subEvents objectForKey:[[subEvents allKeys] objectAtIndex:subFolderIx]];	// List of Rain
+    }
     event = [events objectAtIndex:eventIx];
+    if ([sender isEqual:setCompositeButton])
+        infoDict = [NSDictionary dictionaryWithObject:@"1" forKey:@"composite"];
     [[NSNotificationCenter defaultCenter] postNotificationName:AstroUpdateEvent
-                                                        object:event userInfo:nil];
+                                                        object:event userInfo:infoDict];
 }
 
 
@@ -464,6 +590,7 @@ static int indexFromType(NSString *typeString)
  */
 
 /* load events from file
+ * modified: 2005-07-24
  */
 - (void)loadEvents
 {   NSString		*path;
@@ -475,9 +602,9 @@ static int indexFromType(NSString *typeString)
     [eventBrowser setTarget:self];
     [eventBrowser setAction:@selector(doClick:)];
 
-    path = [NSString stringWithFormat:@"%@/Astro/event.db", userLibrary()];
+    path = vhfPathWithPathComponents(userLibrary(), MODULENAME, @"event.db", nil);
     if ( ![fileManager fileExistsAtPath:path] )
-        path = [NSString stringWithFormat:@"%@/Astro/event.db", localLibrary()];
+        path = vhfPathWithPathComponents(localLibrary(), MODULENAME, @"event.db", nil);
 
     [eventDict release];
     [folders release];
@@ -488,20 +615,28 @@ static int indexFromType(NSString *typeString)
     for (i=0; i<2; i++)
     {
         if (!i)
-            path = [NSString stringWithFormat:@"%@/Astro/Data", localLibrary()];
+            path = vhfPathWithPathComponents(localLibrary(), MODULENAME, @"Data", nil);
         else
-            path = [NSString stringWithFormat:@"%@/Astro/Data", userLibrary()];
+            path = vhfPathWithPathComponents(userLibrary(), MODULENAME, @"Data", nil);
         array = [fileManager directoryContentsAtPath:path];
         for (j=0; j<(int)[array count]; j++)
-        {   NSString	*file = [array objectAtIndex:j];
+        {   NSString	*file     = [array objectAtIndex:j];
+            NSString	*filePath = [path stringByAppendingPathComponent:file];
 
-            if ([file hasSuffix:@"aaf"])
+            if ([file hasSuffix:@"aaf"])	// AAF Archive
             {   NSMutableArray	*dataArray;
 
-                dataArray = [AAFImport dictionaryWithAAFFromFile:[path stringByAppendingPathComponent:file]];
+                dataArray = [AAFImport dictionaryWithAAFFromFile:filePath];
                 [folders addObject:file];
                 [extEventDict setObject:dataArray forKey:file];
             }
+            else if ([file hasSuffix:@"db"])	// Cenon Archive
+            {   NSMutableDictionary	*dataDict;
+
+                dataDict = [NSMutableDictionary dictionaryWithContentsOfFile:filePath];
+                [folders addObject:file];
+                [extEventDict setObject:dataDict forKey:file];
+            }
         }
     }
 
@@ -510,46 +645,86 @@ static int indexFromType(NSString *typeString)
 - (void)saveEvents
 {   NSString	*path;
 
-    path = [NSString stringWithFormat:@"%@/Astro/event.db", userLibrary()];
+    path = vhfPathWithPathComponents(userLibrary(), MODULENAME, @"event.db", nil);
     [eventDict writeToFile:path atomically:YES];
 }
 
-/* delegate methods
+
+/*
+ * delegate methods
+ */
+
+/* browser delegate method
+ * modified: 2005-07-24
  */
 - (int)browser:(NSBrowser*)sender numberOfRowsInColumn:(int)column
-{   int		folderIx;
-    NSArray	*events;
+{   int			folderIx, eventIx;
+    NSArray		*events;
+    NSDictionary	*subEvents;
 
     switch (column)
     {
-        case 0:
+        case 0:		// folders
             return [folders count];
-        case 1:
+        case 1:		// sub-folders or events
             folderIx = [sender selectedRowInColumn:0];
             events = [eventDict objectForKey:[folders objectAtIndex:folderIx]];
             if (!events)
                 events = [extEventDict objectForKey:[folders objectAtIndex:folderIx]];
+            if ([events isKindOfClass:[NSDictionary class]])	// sub-folders
+            {
+                [eventBrowser setHasHorizontalScroller:YES];
+                return [[(NSDictionary*)events allKeys] count];
+            }
+            [eventBrowser setHasHorizontalScroller:NO];
+            return [events count];
+        case 2:		// events in sub-folders
+            folderIx = [sender selectedRowInColumn:0];
+            eventIx  = [sender selectedRowInColumn:1];
+            subEvents = [extEventDict objectForKey:[folders objectAtIndex:folderIx]];		// Met.db
+            events = [subEvents objectForKey:[[subEvents allKeys] objectAtIndex:eventIx]];	// Rain
             return [events count];
     }
     return 0;
 }
 
+/* browser delegate method
+ * modified: 2005-07-24
+ */
 -(void)browser:(NSBrowser*)sender willDisplayCell:(id)cell atRow:(int)row column:(int)column
-{   int		folderIx;
-    NSArray	*events;
+{   int			folderIx, eventIx;
+    NSArray		*events;
+    NSDictionary	*subEvents;
 
     switch (column)
     {
-        case 0:
+        case 0:		// folders
             [cell setStringValue:[folders objectAtIndex:row]];
             [cell setLeaf:NO];
             break;
-        case 1:
+        case 1:		// sub-folders or events
             folderIx = [sender selectedRowInColumn:0];
             events = [eventDict objectForKey:[folders objectAtIndex:folderIx]];
             if (!events)
                 events = [extEventDict objectForKey:[folders objectAtIndex:folderIx]];
-            [cell setStringValue:[[events objectAtIndex:row] objectForKey:@"name"]];
+            if ([events isKindOfClass:[NSDictionary class]])	// sub-folders
+            {   NSArray	*keys = [(NSDictionary*)events allKeys];
+
+                [cell setStringValue:[keys objectAtIndex:row]];
+                [cell setLeaf:NO];
+            }
+            else	// event
+            {
+                [cell setStringValue:[[events objectAtIndex:row] objectForKey:@"name"]];
+                [cell setLeaf:YES];
+            }
+            break;
+        case 2:		// events in sub-folders
+            folderIx = [sender selectedRowInColumn:0];
+            eventIx  = [sender selectedRowInColumn:1];
+            subEvents = [extEventDict objectForKey:[folders objectAtIndex:folderIx]];		// Met.db
+            events = [subEvents objectForKey:[[subEvents allKeys] objectAtIndex:eventIx]];	// List of Rain
+            [cell setStringValue:[[events objectAtIndex:row] objectForKey:@"name"]];		// Rain event
             [cell setLeaf:YES];
     }
 
@@ -580,7 +755,17 @@ static int indexFromType(NSString *typeString)
     events = [eventDict objectForKey:[folders objectAtIndex:folderIx]];
     if (!events)
         events = [extEventDict objectForKey:[folders objectAtIndex:folderIx]];
-    event = [events objectAtIndex:[eventBrowser selectedRowInColumn:1]];
+    if ([events isKindOfClass:[NSDictionary class]])	// sub-folders
+    {   NSDictionary	*subEvents;
+        int		subFolderIx = eventIx;
+
+        eventIx = [eventBrowser selectedRowInColumn:2];
+        if (eventIx < 0)
+            return;
+        subEvents = [extEventDict objectForKey:[folders objectAtIndex:folderIx]];		// Met.db
+        events = [subEvents objectForKey:[[subEvents allKeys] objectAtIndex:subFolderIx]];	// List of Rain
+    }
+    event = [events objectAtIndex:eventIx];
     [eventField setStringValue:[event objectForKey:@"name"]];
     if ( (string = [event stringForKey:@"date"]) )
     {   NSCalendarDate	*date;
@@ -603,10 +788,10 @@ static int indexFromType(NSString *typeString)
     }
     [typePopup selectItemAtIndex:indexFromType([event objectForKey:@"type"])];
     [countryField setStringValue:(string=[event objectForKey:@"country"]) ? string : @""];
-    [cityField setStringValue:(string=[event objectForKey:@"city"]) ? string : @""];
-    [latField setStringValue:(string=[event stringForKey:@"lat"]) ? string : @""];
-    [lonField setStringValue:(string=[event stringForKey:@"lon"]) ? string : @""];
-    [textView setString:(string=[event objectForKey:@"text"]) ? string : @""];
+    [cityField    setStringValue:(string=[event objectForKey:@"city"])    ? string : @""];
+    [latField     setStringValue:(string=[event stringForKey:@"lat"])     ? string : @""];
+    [lonField     setStringValue:(string=[event stringForKey:@"lon"])     ? string : @""];
+    [textView     setString:     (string=[event objectForKey:@"text"])    ? string : @""];
 }
 
 
