@@ -29,7 +29,7 @@
 #ifndef VHF_H_ASTROLOCATIONS
 #define VHF_H_ASTROLOCATIONS
 
-#define MODULENAME	@"Astro"	// the name of the module
+#define MODULENAME	@"Astro"	// the name of the module and name of library folder
 
 /* Map */
 #define	MAP_INFOFILE	@"map.plist"	// predefined regions
